#ifndef _BUCKET_CLIENT_H
#define _BUCKET_CLIENT_H
/*
 * FogLAMP 
 *
 * Copyright (c) 2023 Dianomic Systems
 *
 * Released under the Apache 2.0 Licence
 *
 * Author: Mark Riddoch
 */
#include <management_client.h>
#include <service_record.h>
#include <vector>
#include <config_category.h>
#include <logger.h>
#include <mutex>

#define BUCKET_SERVICE_TYPE	"BucketStorage"
#define BUCKET_ITEM_TYPE	"bucket"

class BucketMatch {
	public:
		BucketMatch(int id, const std::string& file) :
			m_id(id), m_file(file) {}
		int			getId() { return m_id; }
		std::string		getFile() { return m_file; }
	private:
		const int		m_id;
		const std::string	m_file;
};


/**
 * A class for interacting with the Bucket Storage Service
 * 
 * This is a singleton class that may be used by users of
 * the bucket configuration items
 */
class BucketClient {
	public:
		static BucketClient	*getInstance();
		int			matchBucket(const ConfigCategory& config, const std::string& name, std::vector<BucketMatch>& matches);
	public:
		static BucketClient	*instance;
	private:
		BucketClient();
		~BucketClient();
		bool			getServers();
	private:
		ManagementClient	*m_managementClient;
		std::vector<ServiceRecord *>
					m_servers;
		std::mutex		m_mutex;
		Logger			*m_logger;

};
#endif
